@@ -55,7 +55,7 @@
 #include "gstpriv.h"
 #include "re.h"
 
-#include "ffi.h"
+#include "../libffi/include/ffi.h"
 #include <ltdl.h>
 
 typedef struct symbol_type_map
@@ -105,25 +105,15 @@ typedef struct cfunc_info
 }
 cfunc_info;
 
-struct gst_stat_struct
+typedef struct gst_stat
 {
   unsigned short st_mode;	/* protection */
   long st_size;			/* total size, in bytes */
   long st_aTime;		/* time of last access */
   long st_mTime;		/* time of last modification */
   long st_cTime;		/* time of last change */
-};
-
-typedef struct gst_stat
-{
-  OBJ_HEADER;
-  OOP st_mode;		/* protection */
-  OOP st_size;		/* total size, in bytes */
-  OOP st_aTime;		/* time of last access */
-  OOP st_mTime;		/* time of last modification */
-  OOP st_cTime;		/* time of last change */
 }
-*gst_stat;
+gst_stat;
 
 
 
@@ -178,14 +168,10 @@ static OOP classify_type_symbol (OOP symbolOOP,
 static int get_errno (void);
 
 /* Encapsulate binary incompatibilities between various C libraries.  */
-static int my_stat_old (const char *name,
-		        struct gst_stat_struct * out);
-static int my_lstat_old (const char *name,
-		         struct gst_stat_struct * out);
 static int my_stat (const char *name,
-		    OOP out);
+		    gst_stat * out);
 static int my_lstat (const char *name,
-		     OOP out);
+		     gst_stat * out);
 static int my_putenv (const char *str);
 static int my_chdir (const char *str);
 static int my_symlink (const char* oldpath, const char* newpath);
@@ -298,18 +284,12 @@ get_errno (void)
   return (old);
 }
 
-static inline int
-adjust_time (time_t t)
-{
-  return _gst_adjust_time_zone (t) - 86400 * 10957;
-}
-
-static inline int
-my_stat_old (const char *name,
-	     struct gst_stat_struct * out)
+int
+my_stat (const char *name,
+	 gst_stat * out)
 {
   int result;
-  struct stat statOut;
+  static struct stat statOut;
 
   result = stat (name, &statOut);
   if (!result)
@@ -317,41 +297,20 @@ my_stat_old (const char *name,
       errno = 0;
       out->st_mode = statOut.st_mode;
       out->st_size = statOut.st_size;
-      out->st_aTime = adjust_time (statOut.st_atime);
-      out->st_mTime = adjust_time (statOut.st_mtime);
-      out->st_cTime = adjust_time (statOut.st_ctime);
-    }
-  return (result);
-}
-
-int
-my_stat (const char *name,
-	 OOP out)
-{
-  int result;
-  struct stat statOut;
-
-  result = stat (name, &statOut);
-  if (!result)
-    {
-      gst_stat obj = (gst_stat) OOP_TO_OBJ (out);
-      errno = 0;
-      obj->st_mode = FROM_INT (statOut.st_mode);
-      obj->st_aTime = FROM_INT (adjust_time (statOut.st_atime));
-      obj->st_mTime = FROM_INT (adjust_time (statOut.st_mtime));
-      obj->st_cTime = FROM_INT (adjust_time (statOut.st_ctime));
-      obj->st_size = FROM_OFF_T (statOut.st_size);
+      out->st_aTime = _gst_adjust_time_zone (statOut.st_atime) - 86400 * 10957;
+      out->st_mTime = _gst_adjust_time_zone (statOut.st_mtime) - 86400 * 10957;
+      out->st_cTime = _gst_adjust_time_zone (statOut.st_ctime) - 86400 * 10957;
     }
   return (result);
 }
 
 #ifdef HAVE_LSTAT
-static inline int
-my_lstat_old (const char *name,
-	      struct gst_stat_struct * out)
+int
+my_lstat (const char *name,
+	 gst_stat * out)
 {
   int result;
-  struct stat statOut;
+  static struct stat statOut;
 
   result = lstat (name, &statOut);
   if (!result)
@@ -359,36 +318,14 @@ my_lstat_old (const char *name,
       errno = 0;
       out->st_mode = statOut.st_mode;
       out->st_size = statOut.st_size;
-      out->st_aTime = adjust_time (statOut.st_atime);
-      out->st_mTime = adjust_time (statOut.st_mtime);
-      out->st_cTime = adjust_time (statOut.st_ctime);
-    }
-  return (result);
-}
-
-int
-my_lstat (const char *name,
-	 OOP out)
-{
-  int result;
-  struct stat statOut;
-
-  result = lstat (name, &statOut);
-  if (!result)
-    {
-      gst_stat obj = (gst_stat) OOP_TO_OBJ (out);
-      errno = 0;
-      obj->st_mode = FROM_INT (statOut.st_mode);
-      obj->st_aTime = FROM_INT (adjust_time (statOut.st_atime));
-      obj->st_mTime = FROM_INT (adjust_time (statOut.st_mtime));
-      obj->st_cTime = FROM_INT (adjust_time (statOut.st_ctime));
-      obj->st_size = FROM_OFF_T (statOut.st_size);
+      out->st_aTime = _gst_adjust_time_zone (statOut.st_atime) - 86400 * 10957;
+      out->st_mTime = _gst_adjust_time_zone (statOut.st_mtime) - 86400 * 10957;
+      out->st_cTime = _gst_adjust_time_zone (statOut.st_ctime) - 86400 * 10957;
     }
   return (result);
 }
 #else
 #define my_lstat my_stat
-#define my_lstat_old my_stat_old
 #endif
 
 int
@@ -519,11 +456,7 @@ init_dld (void)
 {
   char *modules;
   lt_dlinit ();
-
-  modules = _gst_relocate_path (MODULE_PATH);
-  lt_dladdsearchdir (modules);
-  free (modules);
-
+  lt_dladdsearchdir (MODULE_PATH);
   if ((modules = getenv ("SMALLTALK_MODULES")))
     lt_dladdsearchdir (modules);
 
@@ -556,10 +489,8 @@ _gst_init_cfuncs (void)
 
   _gst_define_cfunc ("errno", get_errno);
   _gst_define_cfunc ("strerror", strerror);
-  _gst_define_cfunc ("stat", my_stat_old);
-  _gst_define_cfunc ("lstat", my_lstat_old);
-  _gst_define_cfunc ("stat_obj", my_stat);
-  _gst_define_cfunc ("lstat_obj", my_lstat);
+  _gst_define_cfunc ("stat", my_stat);
+  _gst_define_cfunc ("lstat", my_lstat);
   _gst_define_cfunc ("utime", _gst_set_file_access_times);
   _gst_define_cfunc ("chmod", chmod);
 
@@ -652,63 +583,6 @@ lookup_function (const char *funcName)
 }
 
 
-int
-_gst_c_type_size (int type)
-{
-  switch (type)
-    {
-    case CDATA_CHAR:
-      return sizeof (char);
-    case CDATA_UCHAR:
-      return sizeof (unsigned char);
-
-    case CDATA_SHORT:
-      return sizeof (short);
-    case CDATA_USHORT:
-      return sizeof (unsigned short);
-
-    case CDATA_INT:
-      return sizeof (int);
-    case CDATA_UINT:
-      return sizeof (unsigned int);
-
-    case CDATA_LONG:
-      return sizeof (long);
-    case CDATA_ULONG:
-      return sizeof (unsigned long);
-
-    case CDATA_FLOAT:
-      return sizeof (float);
-    case CDATA_DOUBLE:
-      return sizeof (double);
-    case CDATA_LONG_DOUBLE:
-      return sizeof (long double);
-
-    case CDATA_OOP:
-      return sizeof (OOP);
-
-    case CDATA_WCHAR:
-      return sizeof (wchar_t);
-
-    case CDATA_WSTRING:
-      return sizeof (wchar_t *);
-
-    case CDATA_STRING:
-    case CDATA_STRING_OUT:
-    case CDATA_SYMBOL:
-    case CDATA_BYTEARRAY:
-    case CDATA_BYTEARRAY_OUT:
-    case CDATA_SYMBOL_OUT:
-      return sizeof (char *);
-
-    case CDATA_COBJECT:
-      return sizeof (void *);
-
-    case CDATA_COBJECT_PTR:
-      return sizeof (void **);
-    }
-}
-
 OOP
 _gst_invoke_croutine (OOP cFuncOOP,
 		      OOP receiver,
@@ -728,11 +602,11 @@ _gst_invoke_croutine (OOP cFuncOOP,
   if (IS_NIL (desc->cFunction))
     return (NULL);
 
-  c_func_cur = (cfunc_info *) cobject_value (desc->cFunction);
+  c_func_cur = (cfunc_info *) COBJECT_VALUE (desc->cFunction);
   if (!c_func_cur)
     return (NULL);
 
-  fixedArgs = NUM_INDEXABLE_FIELDS (cFuncOOP);
+  fixedArgs = TO_INT (desc->numFixedArgs);
   totalArgs = 0;
   for (si = i = 0; i < fixedArgs; i++)
     {
@@ -809,7 +683,7 @@ _gst_invoke_croutine (OOP cFuncOOP,
         switch (arg->cType)
           {
           case CDATA_COBJECT_PTR:
-	    set_cobject_value (arg->oop, arg->u.cObjectPtrVal.ptrVal);
+	    SET_COBJECT_VALUE (arg->oop, arg->u.cObjectPtrVal.ptrVal);
 	    continue;
 
           case CDATA_WSTRING_OUT:
@@ -1110,13 +984,13 @@ push_smalltalk_obj (OOP oop,
 	  /* Set up an indirect pointer to protect against the OOP
 	     moving during the call-out.  */
 	  cp->u.cObjectPtrVal.pPtrVal = &cp->u.cObjectPtrVal.ptrVal;
-	  cp->u.cObjectPtrVal.ptrVal = cobject_value (oop);
+	  cp->u.cObjectPtrVal.ptrVal = COBJECT_VALUE (oop);
 	  cp->oop = oop;
 	  SET_TYPE (&ffi_type_pointer);
 	  return;
 
 	case CDATA_COBJECT:
-	  cp->u.ptrVal = cobject_value (oop);
+	  cp->u.ptrVal = COBJECT_VALUE (oop);
 	  SET_TYPE (&ffi_type_pointer);
 	  return;
 	}
@@ -1219,8 +1093,8 @@ c_to_smalltalk (cparam *result, OOP returnTypeOOP)
 	{
 	  if (IS_INT (returnTypeOOP))
 	    returnTypeOOP = _gst_nil_oop;
-	  resultOOP = COBJECT_NEW (result->u.ptrVal, returnTypeOOP,
-				   _gst_c_object_class);
+	  resultOOP = _gst_c_object_new (result->u.ptrVal, returnTypeOOP,
+					 _gst_c_object_class);
 	}
       else if (returnType == CDATA_STRING || returnType == CDATA_STRING_OUT)
 	{
@@ -1275,8 +1149,7 @@ bad_type (OOP class_oop,
 
 
 OOP
-_gst_make_descriptor (OOP classOOP,
-		      OOP funcNameOOP,
+_gst_make_descriptor (OOP funcNameOOP,
 		      OOP returnTypeOOP,
 		      OOP argsOOP)
 {
@@ -1301,38 +1174,27 @@ _gst_make_descriptor (OOP classOOP,
      OOPs */
   incPtr = INC_SAVE_POINTER ();
 
-  cFunction = COBJECT_NEW (cfi, _gst_nil_oop, _gst_c_object_class);
+  cFunction = COBJECT_NEW (cfi);
   INC_ADD_OOP (cFunction);
 
   cFunctionName = _gst_string_new (funcName);
-  xfree (funcName);
   INC_ADD_OOP (cFunctionName);
 
-  desc = (gst_cfunc_descriptor) new_instance_with (classOOP, numArgs, &descOOP);
+  desc = (gst_cfunc_descriptor)
+    new_instance_with (_gst_c_func_descriptor_class, numArgs,
+		       &descOOP);
 
   desc->cFunction = cFunction;
   desc->cFunctionName = cFunctionName;
-  desc->tagOOP = _gst_nil_oop;
+  desc->numFixedArgs = FROM_INT (numArgs);
   desc->returnType = classify_type_symbol (returnTypeOOP, true);
-  if (desc->returnType == _gst_nil_oop)
-    goto error;
-
   for (i = 1; i <= numArgs; i++)
-    {
-      OOP type;
-      type = desc->argTypes[i - 1] =
-        classify_type_symbol (ARRAY_AT (argsOOP, i), false);
-      if (type == _gst_nil_oop)
-	goto error;
-    }
+    desc->argTypes[i - 1] =
+      classify_type_symbol (ARRAY_AT (argsOOP, i), false);
 
+  xfree (funcName);
   INC_RESTORE_POINTER (incPtr);
   return (descOOP);
-
- error:
-  INC_RESTORE_POINTER (incPtr);
-  return (NULL);
-
 }
 
 OOP
@@ -1340,6 +1202,8 @@ classify_type_symbol (OOP symbolOOP,
 		      mst_Boolean isReturn)
 {
   const symbol_type_map *sp;
+  char *symbolName;
+
   for (sp = type_map; sp->symbol != NULL; sp++)
     {
       if (*sp->symbol == symbolOOP)
@@ -1352,7 +1216,14 @@ classify_type_symbol (OOP symbolOOP,
 	return (symbolOOP);	/* this is the type we want! */
     }
 
-  return _gst_nil_oop;
+  symbolName = _gst_to_cstring (symbolOOP);	/* yeah, yeah...but
+						   they have the same
+						   representation! */
+  _gst_errorf ("Unknown data type symbol: %s", symbolName);
+
+  xfree (symbolName);
+
+  return (FROM_INT (CDATA_UNKNOWN));
 }
 
 void
