@@ -90,6 +90,9 @@ const char *_gst_image_file_path = NULL;
    home directory.  */
 const char *_gst_user_file_base_path = NULL;
 
+/* The path to the executable, derived from argv[0].  */
+const char *_gst_executable_path = NULL;
+
 /* Whether to look for user files.  */
 static mst_Boolean no_user_files = false;
 
@@ -228,14 +231,14 @@ static const char standard_files[] = {
   "ProcSched.st\0"
   "Delay.st\0"
   "SharedQueue.st\0"
-  "ContextPart.st\0"
-  "MthContext.st\0"
-  "BlkContext.st\0"
-  "BlkClosure.st\0"
   "Behavior.st\0"
   "ClassDesc.st\0"
   "Class.st\0"
   "Metaclass.st\0"
+  "ContextPart.st\0"
+  "MthContext.st\0"
+  "BlkContext.st\0"
+  "BlkClosure.st\0"
   "Continuation.st\0"
   "Memory.st\0"
   "MethodInfo.st\0"
@@ -264,13 +267,11 @@ static const char standard_files[] = {
   "CFuncs.st\0"
   "CStruct.st\0"
 
-  /* Exception handling and ProcessEnvironment */
-  "ProcEnv.st\0"
+  /* Exception handling */
   "ExcHandling.st\0"
   "AnsiExcept.st\0"
 
   /* Virtual filesystem layer */
-  "FilePath.st\0"
   "File.st\0"
   "Directory.st\0"
   "VFS.st\0"
@@ -307,6 +308,7 @@ _gst_smalltalk_args (int argc,
 {
   smalltalk_argc = argc;
   smalltalk_argv = argv;
+  _gst_executable_path = _gst_find_executable (argv[0]);
 }
 
    
@@ -328,9 +330,6 @@ _gst_initialize (const char *kernel_dir,
   _gst_smalltalk_initialized = true;
   _gst_init_snprintfv ();
 
-  if (!_gst_executable_path)
-    _gst_executable_path = DEFAULT_EXECUTABLE;
-
   /* By default, apply this kludge fpr OSes such as Windows and MS-DOS
      which have no concept of home directories.  */
   if (home == NULL)
@@ -353,8 +352,6 @@ _gst_initialize (const char *kernel_dir,
 	}
     }
 
-  /* For the image file, it is okay to find none if we can/should rebuild
-     the image file.  */
   if (image_file
       && (flags & (GST_REBUILD_IMAGE | GST_MAYBE_REBUILD_IMAGE)) == 0
       && !_gst_file_is_readable (image_file))
@@ -368,8 +365,6 @@ _gst_initialize (const char *kernel_dir,
 	}
     }
 
-  /* The image path can be used as the default kernel path, so we split
-     it anyway into directory+filename.  */
   if (image_file)
     {
       const char *p;
@@ -400,31 +395,26 @@ _gst_initialize (const char *kernel_dir,
     }
   else
     {
-      /* No image file given, we use the system default or revert to the
+      /* No image file given, we use the system default but revert to the
 	 current directory.  */
-      str = _gst_relocate_path (IMAGE_PATH);
-      if (_gst_file_is_readable (str))
-        _gst_image_file_path = str;
+      if (_gst_file_is_readable (IMAGE_PATH))
+        _gst_image_file_path = IMAGE_PATH;
       else
-	{
-          free (str);
-          _gst_image_file_path = xstrdup (currentDirectory);
-	}
-
+        _gst_image_file_path = xstrdup (currentDirectory);
       flags |= GST_IGNORE_BAD_IMAGE_PATH;
       image_file = "gst.im";
     }
 
   if (!kernel_dir)
     {
-      str = _gst_relocate_path (KERNEL_PATH);
-      if (!_gst_file_is_readable (str))
+      if (_gst_file_is_readable (KERNEL_PATH))
+        kernel_dir = KERNEL_PATH;
+      else
 	{
-          free (str);
-	  asprintf (&str, "%s/kernel", _gst_image_file_path);
+          char *kernel_file_path;
+	  asprintf (&kernel_file_path, "%s/kernel", _gst_image_file_path);
+	  kernel_dir = kernel_file_path;
 	}
-
-      kernel_dir = str;
     }
 
   xfree (currentDirectory);
@@ -580,8 +570,6 @@ load_standard_files (void)
 	}
     }
 
-  _gst_msg_sendf (NULL, "%v %o relocate", _gst_file_segment_class);
-
   if (site_pre_image_file)
     _gst_process_file (site_pre_image_file, GST_DIR_ABS);
 
